//
//  BKTracingView.h
//  BKTracingView
//
//  Created by boreal-kiss.com on 11/01/29.
//  Copyright 2011 boreal-kiss.com. All rights reserved.
//

#import <UIKit/UIKit.h>

/**
 * The UIView subclass for tracing animations.
 * Each frame animation up to the current frame will stay displayed until the clear method is explicitly called.
 */
@interface BKTracingView : UIView {
	UIImage *_bkImage;
	NSTimer *_bkTimer;
	NSTimeInterval _bkTimerInterval;
}
/**
 * Animation timing.
 */
@property (nonatomic, assign) NSTimeInterval timerInterval;

/**
 * This method will be called with a specific animation timing.
 * The argument corresponds to the current graphics context for CoreGraphics.
 */
-(void)drawContext:(CGContextRef)context;

/**
 * Clears the current content.
 */
-(void)clearContext:(CGContextRef)context;

/**
 * When the animation starts, the drawing method will be called with a specific animation timing.
 */
-(void)startAnimation:(id)sender;

/**
 * The animation stops.
 */
-(void)stopAnimation:(id)sender;
@end
