//
//  BKTracingView.m
//  BKTracingView
//
//  Created by boreal-kiss.com on 11/01/29.
//  Copyright 2011 boreal-kiss.com. All rights reserved.
//

#import "BKTracingView.h"

//Private
static const NSTimeInterval DefaultTimerInterval = 1.0 / 20.0;

//Private
@interface BKTracingView ()
@property (nonatomic, retain) UIImage *bkImage;
@property (nonatomic, retain) NSTimer *bkTimer;
-(void)_bkSetup;
-(void)_bkStartTimer;
-(void)_bkStopTimer;
-(void)_bkTimerHandler:(NSTimer *)aTimer;
-(void)_bkDrawImage:(UIImage *)anImage context:(CGContextRef)context;
@end

@implementation BKTracingView
@synthesize bkImage = _bkImage;
@synthesize bkTimer = _bkTimer;
@synthesize timerInterval = _bkTimerInterval;

-(id)initWithCoder:(NSCoder *)aDecoder{
	if (self = [super initWithCoder:aDecoder]){
		[self _bkSetup];
	}
	return self;
}

-(id)initWithFrame:(CGRect)aFrame{
	if (self = [super initWithFrame:aFrame]){
		[self _bkSetup];
	}
	return self;
}

-(void)drawContext:(CGContextRef)context{
	//Must be overridden by a user.
}

-(void)clearContext:(CGContextRef)context{
	CGContextClearRect(context, self.bounds);
}

-(void)startAnimation:(id)sender{
	[self _bkStartTimer];
}

-(void)stopAnimation:(id)sender{
	[self _bkStopTimer];
}

- (void)drawRect:(CGRect)rect {
	if (_bkImage){
		[_bkImage drawInRect:rect];
	}
}

- (void)dealloc {
	[self _bkStopTimer];
	self.bkImage = nil;
    [super dealloc];
}

#pragma mark -
#pragma mark Private

-(void)_bkSetup{
	_bkTimerInterval = DefaultTimerInterval;
}

-(void)_bkStartTimer{
	self.bkTimer = [NSTimer scheduledTimerWithTimeInterval:_bkTimerInterval target:self selector:@selector(_bkTimerHandler:) userInfo:nil repeats:YES];
}

-(void)_bkStopTimer{
	if (_bkImage){
		[_bkTimer invalidate];
		self.bkTimer = nil;
	}
}

-(void)_bkTimerHandler:(NSTimer *)aTimer{
	CGFloat w = self.frame.size.width;
	CGFloat h = self.frame.size.height;
	
	UIGraphicsBeginImageContext(CGSizeMake(w, h));
	CGContextRef context = UIGraphicsGetCurrentContext();
	
	//Draws the image cached in the previous frame.
	[self _bkDrawImage:_bkImage context:context];
	
	//User's drawing method.
	[self drawContext:context];
	
	//Stores the drawing in a new image.
	self.bkImage = UIGraphicsGetImageFromCurrentImageContext();
	
	UIGraphicsEndImageContext();
}

-(void)_bkDrawImage:(UIImage *)anImage context:(CGContextRef)context{
	CGFloat w = self.frame.size.width;
	CGFloat h = self.frame.size.height;
	
	//Configures the image in the opposite direction along the y-axis,
	//because the CoreGraphics has the different coordinates from the UIKit.
	CGContextSaveGState(context);
	CGContextScaleCTM(context, 1.0, -1.0);
	CGContextTranslateCTM(context, 0.0, -1.0 * h);
	CGContextDrawImage(context, CGRectMake(0, 0, w, h), anImage.CGImage);
	CGContextRestoreGState(context);
}

#pragma mark -
#pragma mark Accessors

-(void)setBkImage:(UIImage *)anImage{
	if (_bkImage != anImage){
		[_bkImage release];
		_bkImage = [anImage retain];
		[self setNeedsDisplay];
	}
}

@end
