//
//  ButterflyCurvesView.m
//  BKTracingView
//
//  Created by boreal-kiss.com on 11/01/29.
//  Copyright 2011 boreal-kiss.com. All rights reserved.
//

#import "ButterflyCurvesView.h"

//Private
static const CGFloat ScaleFactor = 40.0;

//Private
@interface ButterflyCurvesView ()
-(void)_drawCurvesForTheta:(CGFloat)theta context:(CGContextRef)context;
-(void)_drawCircleAtPoint:(CGPoint)center radius:(CGFloat)radius context:(CGContextRef)context;
-(UIColor *)_colorAtPoint:(CGPoint)aPoint;
@end

@implementation ButterflyCurvesView

//Override
-(void)drawContext:(CGContextRef)context{	
	static float theta = 0.0;
	theta += 0.001 * M_PI;
	[self _drawCurvesForTheta:theta context:context];
}

#pragma mark -
#pragma mark Private

-(void)_drawCurvesForTheta:(CGFloat)theta context:(CGContextRef)context{
	CGFloat w = self.frame.size.width;
	CGFloat h = self.frame.size.height;
	
	CGPoint theCenter = CGPointMake(0.5 * w, 0.5 * h);
	
	CGFloat r = exp(cos(theta)) - 2 * cos(4 * theta) + pow(sin(theta / 12), 5);
	CGFloat x = r * sin(theta);
	CGFloat y = r * cos(theta);
	CGFloat xx = theCenter.x + ScaleFactor * x;
	CGFloat yy = theCenter.y + ScaleFactor * (-y);
	CGPoint p = CGPointMake(xx, yy);
	
	UIColor *color = [self _colorAtPoint:p];
	CGContextSetFillColorWithColor(context, color.CGColor);
	[self _drawCircleAtPoint:p radius:1 context:context];
}

-(void)_drawCircleAtPoint:(CGPoint)center radius:(CGFloat)radius context:(CGContextRef)context{
	CGContextAddArc(context, center.x, center.y, radius, 0, 2 * M_PI, 1);
	CGContextDrawPath(context, kCGPathFill);
}

-(UIColor *)_colorAtPoint:(CGPoint)aPoint{
	float r = 0.0;
	float g = aPoint.x / self.frame.size.width;
	float b = aPoint.y / self.frame.size.height;
	return [UIColor colorWithRed:r green:g blue:b alpha:1.0];
}

@end