//
//  IkedaAtractorView.m
//  BKTracingView
//
//  Created by boreal-kiss.com on 11/01/30.
//  Copyright 2011 boreal-kiss.com. All rights reserved.
//

#import "IkedaAtractorView.h"

//Private
static const float C1	= 0.4;
static const float C2	= 0.9;
static const float C3	= 6.0;
static const float Rho	= 1.0;
static const float Scale = 100.0;

//Private
@interface IkedaAtractorView ()
//-(void)_drawCurvesForX:(CGFloat)x y:(CGFloat)y context:(CGContextRef)context;
-(void)_drawCurvesForX:(float *)xp y:(float *)yp context:(CGContextRef)context;

-(void)_drawCircleAtPoint:(CGPoint)center radius:(CGFloat)radius context:(CGContextRef)context;
-(UIColor *)_colorAtPoint:(CGPoint)aPoint;
@end

@implementation IkedaAtractorView

//Override
-(void)drawContext:(CGContextRef)context{
	//NSLog(@"%s", __FUNCTION__);
	//[self clearContext:context];
	
	static float x = 0.0;
	static float y = 0.0;
	
	[self _drawCurvesForX:&x y:&y context:context];
}

#pragma mark -
#pragma mark Private

-(void)_drawCurvesForX:(float *)xp y:(float *)yp context:(CGContextRef)context{	
	float theta = C1 - C3 / (1.0 + (*xp) * (*xp) + (*yp) * (*yp));
	float co = cos(theta);
	float si = sin(theta);
	float xt = Rho + C2 * ((*xp) * co - (*yp) * si);
	(*yp) = C2 * ((*xp) * si + (*yp) * co);
	(*xp) = xt;
	
	CGFloat w = self.frame.size.width;
	CGFloat h = self.frame.size.height;
	CGPoint theCenter = CGPointMake(0.5 * w, 0.5 * h);
	CGFloat xx = theCenter.x + Scale * (*xp);
	CGFloat yy = theCenter.y + Scale * (*yp);
	CGPoint p = CGPointMake(xx, yy);
	UIColor *color = [self _colorAtPoint:p];
	CGContextSetFillColorWithColor(context, color.CGColor);
	[self _drawCircleAtPoint:p radius:1 context:context];
}

-(void)_drawCircleAtPoint:(CGPoint)center radius:(CGFloat)radius context:(CGContextRef)context{
	CGContextAddArc(context, center.x, center.y, radius, 0, 2 * M_PI, 1);
	CGContextDrawPath(context, kCGPathFill);
}

-(UIColor *)_colorAtPoint:(CGPoint)aPoint{
	float r = 0.0;
	float g = aPoint.x / self.frame.size.width;
	float b = aPoint.y / self.frame.size.height;
	return [UIColor colorWithRed:r green:g blue:b alpha:1.0];
}


@end
