//
//  LorentzAttractorView.m
//  BKTracingView
//
//  Created by boreal-kiss.com on 11/01/30.
//  Copyright 2011 boreal-kiss.com. All rights reserved.
//

#import "LorentzAttractorView.h"

//Private
static const float kDump	= 0.006;
static const float kScale	= 7.0;

//Private
@interface LorentzAttractorView ()
-(void)_drawForX:(float *)xp y:(float *)yp z:(float *)zp context:(CGContextRef)context;
-(void)_drawCircleAtPoint:(CGPoint)center radius:(CGFloat)radius context:(CGContextRef)context;
-(UIColor *)_colorAtPoint:(CGPoint)aPoint;
@end

@implementation LorentzAttractorView

//Override
-(void)drawContext:(CGContextRef)context{
	static float x = 0.6;
	static float y = 0.6;
	static float z = 0.6;
	[self _drawForX:&x y:&y z:&z context:context];
}

#pragma mark -
#pragma mark Private

-(void)_drawForX:(float *)xp y:(float *)yp z:(float *)zp context:(CGContextRef)context{	
	float x = (*xp);
	float y = (*yp);
	float z = (*zp);
	
	float newX = x + kDump * 10.0 * (y - x);
	float newY = y + kDump * (28.0 * x - x * z - y);
	float newZ = z + kDump * (x * y - 8.0 * z / 3.0);
	
	*xp = newX;
	*yp = newY;
	*zp = newZ;
	
	CGFloat w = self.frame.size.width;
	CGFloat h = self.frame.size.height;
	CGPoint theCenter = CGPointMake(0.5 * w, 0.5 * h);
	CGFloat xx = theCenter.x + kScale * newX;
	CGFloat yy = theCenter.y + kScale * newY;
	CGPoint p = CGPointMake(xx, yy);
	UIColor *color = [self _colorAtPoint:p];
	CGContextSetFillColorWithColor(context, color.CGColor);
	[self _drawCircleAtPoint:p radius:1 context:context];
}

-(void)_drawCircleAtPoint:(CGPoint)center radius:(CGFloat)radius context:(CGContextRef)context{
	CGContextAddArc(context, center.x, center.y, radius, 0, 2 * M_PI, 1);
	CGContextDrawPath(context, kCGPathFill);
}

-(UIColor *)_colorAtPoint:(CGPoint)aPoint{
	float r = 0.0;
	float g = aPoint.x / self.frame.size.width;
	float b = aPoint.y / self.frame.size.height;
	return [UIColor colorWithRed:r green:g blue:b alpha:1.0];
}

@end
